#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import unittest

from mayavi.core.ui.api import MlabSceneModel

from traits_enaml.testing.enaml_test_assistant import EnamlTestAssistant


class MayaviCanvasTestCase(EnamlTestAssistant, unittest.TestCase):

    def setUp(self):

        EnamlTestAssistant.setUp(self)

        enaml_source = """
from enaml.widgets.api import MainWindow
from traits_enaml.widgets.mayavi_canvas import MayaviCanvas

enamldef MainView(MainWindow):
    attr scene

    MayaviCanvas:
        scene << parent.scene

"""
        self.scene = MlabSceneModel()
        self.view, _ = self.parse_and_create(enaml_source, scene=self.scene)
        self.canvas = self.find_enaml_widget(self.view, 'MayaviCanvas')

    def tearDown(self):
        self.canvas = None
        self.view = None
        self.scene = None
        EnamlTestAssistant.tearDown(self)

    def test_updating_the_scene(self):
        canvas = self.canvas

        with self.assertTraitChanges(canvas.model, 'scene', count=1):
            canvas.scene = MlabSceneModel()
        self.assertEqual(canvas.model.scene, canvas.scene)

    def test_toggling_the_mayavi_toolbar(self):
        canvas = self.canvas
        editor = canvas.ui.get_editors('scene')[0]
        toolbar = editor._scene._tool_bar

        # show the view
        with self.event_loop():
            self.view.show()

        # toggle the toolbar
        with self.event_loop():
            canvas.show_toolbar = False
        self.assertFalse(toolbar.isVisible())
        with self.event_loop():
            canvas.show_toolbar = True
        self.assertTrue(toolbar.isVisible())


if __name__ == "__main__":
    unittest.main()



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
from collections import namedtuple

from traits.testing.unittest_tools import reverse_assertion


Change = namedtuple(
    'Change',
    ['object', 'type', 'name', 'operation', 'item', 'oldvalue', 'value'])


class _AssertAtomChangesContext(object):
    """ Context manager used to implement TestAssistant.assertAtomChanges.

    """
    def __init__(self, obj, xname, count, test_case):
        self.obj = obj
        self.xname = xname
        self.count = count
        self.event = None
        self.events = []
        self.failureException = test_case.failureException

    def _listener(self, change):
        """ Dummy atom listener.

        """
        obj = change['object']
        type_ = change['type']
        name = change['name']
        value = change['value']
        old_value = change.get('oldvalue', None)
        operation = change.get('operation', None)
        item = change.get('item', None)
        self.event = Change(
            obj, type_, name, operation, item, old_value, value)
        self.events.append(self.event)

    def __enter__(self):
        """ Bind the trait listener.

        """
        self.obj.observe(self.xname, self._listener)
        return self

    def __exit__(self, exc_type, exc_value, tb):
        """ Remove the trait listener.

        """
        if exc_type is not None:
            return False

        self.obj.unobserve(self.xname)
        if self.event is None and self.count != 0:
            msg = 'A change event was not fired for: {0}'.format(self.xname)
            raise self.failureException(msg)
        elif self.count is not None and len(self.events) != self.count:
            msg = 'Change event for {0} was fired {1} times instead of {2}'
            items = self.xname, len(self.events), self.count
            raise self.failureException(msg.format(*items))

        return False


class AtomTestAssistant(object):
    """ Mixin class to augment the unittest.TestCase with useful atom
    assert methods.

    """

    ### Trait assertion methods ########################################

    def assertAtomChanges(
            self, obj, trait, count=None, callableObj=None, *args, **kwargs):
        """ Same method as assertTraitChanges but for Atom based object ...

        """

        context = _AssertAtomChangesContext(obj, trait, count, self)
        if callableObj is None:
            return context
        with context:
            callableObj(*args, **kwargs)

    def assertAtomDoesNotChange(
            self, obj, atom, callableObj=None, *args, **kwargs):
        """Assert an object atom does not change.

        Assert that the class atom does not change during execution of the
        provided function.

        """
        msg = 'A change event was fired for: {0}'.format(atom)
        context = _AssertAtomChangesContext(obj, atom, None, self)
        if callableObj is None:
            return reverse_assertion(context, msg)
        with reverse_assertion(context, msg):
            callableObj(*args, **kwargs)



#------------------------------------------------------------------------------
# Copyright (c) 2005-2013, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in /LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#------------------------------------------------------------------------------
import unittest

from atom.api import Atom, Bool, Float, ContainerList, observe

from traits_enaml.testing.atom_test_assistant import AtomTestAssistant


class TestObject(Atom):

    number = Float(2.0)
    list_of_numbers = ContainerList(Float())
    flag = Bool()

    @observe('number')
    def _add_number_to_list(self, change):
        self.list_of_numbers.append(change['value'])

    def add_to_number(self, value):
        self.number += value


class TestAtomTestAssistant(unittest.TestCase, AtomTestAssistant):

    def setUp(self):
        self.maxDiff = None
        self.test_object = TestObject()

    def test_when_using_with(self):
        """ Check normal use cases as a context manager.
        """
        test_object = self.test_object

        # default values are lazy
        test_object.number

        # Change event should NOT BE detected
        with self.assertAtomDoesNotChange(test_object, 'number') as result:
            test_object.flag = True
            test_object.number = 2.0

        msg = 'The assertion result is not None: {0}'.format(result.event)
        self.assertIsNone(result.event, msg=msg)

        # Change event should BE detected
        with self.assertAtomChanges(test_object, 'number') as result:
            test_object.flag = False
            test_object.number = 5.0

        expected = (test_object, 'update', 'number', None, None, 2.0, 5.0)
        self.assertSequenceEqual(expected, result.event)

        # Change event should BE detected exactly 2 times
        with self.assertAtomChanges(test_object, 'number', count=2) as result:
            test_object.flag = False
            test_object.number = 4.0
            test_object.number = 3.0

        expected = [(test_object, 'update', 'number', None, None, 5.0, 4.0),
                    (test_object, 'update', 'number', None, None, 4.0, 3.0)]
        self.assertSequenceEqual(expected, result.events)
        self.assertSequenceEqual(expected[-1], result.event)

        # Change event should BE detected
        with self.assertAtomChanges(test_object, 'number') as result:
            test_object.flag = True
            test_object.add_to_number(10.0)

        expected = (test_object, 'update', 'number', None, None, 3.0, 13.0)
        self.assertSequenceEqual(expected, result.event)

        # Change event should BE detected exactly 3 times
        with self.assertAtomChanges(test_object, 'number', count=3) as result:
            test_object.flag = True
            test_object.add_to_number(10.0)
            test_object.add_to_number(10.0)
            test_object.add_to_number(10.0)

        expected = [
            (test_object, 'update', 'number', None, None, 13.0, 23.0),
            (test_object, 'update', 'number', None, None, 23.0, 33.0),
            (test_object, 'update', 'number', None, None, 33.0, 43.0)]
        self.assertSequenceEqual(expected, result.events)
        self.assertSequenceEqual(expected[-1], result.event)

    def test_when_using_functions(self):
        test_object = self.test_object

        # Change event should BE detected (including the initial creation)
        self.assertAtomChanges(
            test_object, 'number', 2, test_object.add_to_number, 13.0)

        # default values are lazy
        test_object.flag
        # Change event should NOT BE detected
        self.assertAtomDoesNotChange(
            test_object, 'flag', test_object.add_to_number, 13.0)

    def test_indirect_events(self):
        """ Check catching indirect change events.
        """
        test_object = self.test_object

        # Change event should BE detected
        with self.assertAtomChanges(
                test_object, 'list_of_numbers') as result:
            test_object.flag = True
            test_object.number = -3.0

        expected = (
            test_object, 'container', 'list_of_numbers',
            'append', -3.0,  None, [-3.0])
        self.assertSequenceEqual(expected, result.event)

    def test_exception_inside_context(self):
        """ Check that exception inside the context statement block are
        propagated.

        """
        test_object = self.test_object

        with self.assertRaises(AttributeError):
            with self.assertAtomChanges(test_object, 'number'):
                test_object.i_do_exist

        with self.assertRaises(AttributeError):
            with self.assertAtomDoesNotChange(test_object, 'number'):
                test_object.i_do_exist

    def test_non_change_on_failure(self):
        """ Check behaviour when assertion should be raised for non trait
        change.

        """
        test_object = self.test_object
        # default values are lazy
        test_object.number

        atoms = 'flag', 'number'
        with self.assertRaises(AssertionError):
            with self.assertAtomDoesNotChange(test_object, atoms) as result:
                test_object.flag = True
                test_object.number = -3.0
        expected = [
            (test_object, 'create', 'flag', None, None, None, True),
            (test_object, 'update', 'number', None, None, 2.0, -3.0)]
        self.assertEqual(result.events, expected)

    def test_change_on_failure(self):
        """ Check behaviour when assertion should be raised for trait change.
        """
        test_object = self.test_object
        with self.assertRaises(AssertionError):
            with self.assertAtomChanges(test_object, 'number') as result:
                test_object.flag = True
        self.assertEqual(result.events, [])

        # Change event will not be fired 3 times
        with self.assertRaises(AssertionError):
            with self.assertAtomChanges(
                    test_object, 'number', count=4) as result:
                test_object.flag = True
                test_object.add_to_number(10.0)
                test_object.add_to_number(10.0)

        expected = [
            (test_object, 'create', 'number', None, None, None, 2.0),
            (test_object, 'update', 'number', None, None, 2.0, 12.0),
            (test_object, 'update', 'number', None, None, 12.0, 22.0)]
        self.assertSequenceEqual(expected, result.events)

    def test_asserts_in_context_block(self):
        """ Make sure that the atoms context manager does not stop
        regular assertions inside the managed code block from happening.
        """
        test_object = TestObject(number=16.0)

        with self.assertAtomDoesNotChange(test_object, 'number'):
            self.assertEqual(test_object.number, 16.0)

        with self.assertRaisesRegexp(AssertionError, '16\.0 != 12\.0'):
            with self.assertAtomDoesNotChange(test_object, 'number'):
                self.assertEqual(test_object.number, 12.0)

    def test_special_case_for_count(self):
        """ Count equal to 0 should be valid but it is discouraged.
        """
        test_object = TestObject(number=16.0)

        with self.assertAtomChanges(test_object, 'number', count=0):
            test_object.flag = True


if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------------
#
#  Copyright (c) 2012-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import types

from enaml.core.parser import parse
from enaml.core.enaml_compiler import EnamlCompiler

import traits_enaml

from .gui_test_assistant import GuiTestAssistant
from .atom_test_assistant import AtomTestAssistant


def print_enaml_widget_tree(widget, level=0):
    """ Debugging helper to print out the enaml widget tree starting at a
    particular `widget`.

    Parameters
    ----------
    widget: :class:`~enaml.core.object.Object`
        The root widget in the tree to print.
    level: int
        The current level in the tree. Used internally for displaying the
        tree level.

    """
    level = level + 4
    if level == 0:
        print
    print ' '*level, widget
    for child in widget.children:
        print_enaml_widget_tree(child, level=level)
    if level == 0:
        print


class EnamlTestAssistant(GuiTestAssistant, AtomTestAssistant):

    def tearDown(self):
        super(EnamlTestAssistant, self).tearDown()
        self.enaml_module = None

    def find_toolkit_widget(self, root, type_name):
        """ A simple function that recursively walks a widget tree until it
        finds a widget of a particular type.

        """

        if type_name in [cls.__name__ for cls in type(root).__mro__]:
            return root.widget

        for child in root.children():
            found = self.find_toolkit_widget(child, type_name)
            if found is not None:
                return found

        return None

    def find_all_enaml_widgets(self, root, type_name):
        """ A simple function that recursively walks a widget tree and returns
        a list of all widgets of a particular type.

        """

        return [child for child in root.traverse()
                if type_name in [cls.__name__ for cls in type(child).__mro__]]

    def find_enaml_widget(self, root, type_name):
        """ A simple function that recursively walks a widget tree until it
        finds a widget of a particular type.

        """

        for child in root.traverse():
            if type_name in [cls.__name__ for cls in type(child).__mro__]:
                return child

        return None

    def parse_and_create(self, source, **kwargs):
        """ Parses and compiles the source. The source should have a
        component defined with the name 'MainView'.

        Arguments
        ---------
        source : str
            The enaml source file

        kwargs : dict
            The default attribute values to pass to the component.

        Returns
        -------
            A tuple of the server and client component trees for the
            'MainView' component.

        """

        # This replicates what enaml.runner.main does
        enaml_ast = parse(source, filename='__enaml_tests__')
        code = EnamlCompiler.compile(enaml_ast, '__enaml_tests__')

        enaml_module = types.ModuleType('__tests__')
        ns = enaml_module.__dict__

        with traits_enaml.imports():
            exec code in ns
        View = ns['MainView']

        enaml_view = View(**kwargs)
        enaml_view.initialize()
        if not enaml_view.proxy_is_active:
            enaml_view.activate_proxy()

        toolkit_view = enaml_view.proxy

        # We need to keep a reference to the enaml_module to ensure it does not
        # get collected. If no reference is kept, enaml operators will not be
        # able to resolve objects properly (eg. validator = IntValidator(...)
        # will fail)
        self.enaml_module = enaml_module

        return enaml_view, toolkit_view



# Copyright (c) 2012 by Enthought Inc.
import StringIO
import sys
import time
import unittest

from enaml.application import deferred_call
from enaml.qt import QtGui, QtCore

from traits.api import Float, HasTraits, Int, List, on_trait_change
from traits.testing.unittest_tools import reverse_assertion
from traits_enaml.testing.gui_test_assistant import (
    GuiTestAssistant, print_qt_widget_tree)


class MyClass(HasTraits):

    number = Float(2.0)
    list_of_numbers = List(Float)

    @on_trait_change('number')
    def _add_number_to_list(self, value):
        self.list_of_numbers.append(value)

    def add_to_number(self, value):
        self.number += value


class EventLoopUser(HasTraits):

    value = Int(0)

    maximum_value = Int(15)

    def increase_value(self):
        time.sleep(0.01)
        self.value += 1
        if self.value < self.maximum_value:
            deferred_call(self.increase_value)


class TestGuiTestAssistant(GuiTestAssistant, unittest.TestCase):

    def setUp(self):
        self.my_class = MyClass()
        GuiTestAssistant.setUp(self)

    def _set_trait(self, value):
        self.my_class.number = value

    def test_trait_change_on_event_loop(self):
        condition = lambda obj: obj.number == 5.0
        with self.assertTraitChangesInEventLoop(
                self.my_class, 'number', condition) as collector:
            deferred_call(self._set_trait, 5.0)
        self.assertEqual(collector.event_count, 1)

    def test_timeout(self):
        condition = lambda obj: obj.number == 3.0
        with self.assertRaises(AssertionError):
            with self.assertTraitChangesInEventLoop(
                    self.my_class, 'number', condition, timeout=0.5):
                deferred_call(self._set_trait, 2.0)

    def test_exception_in_context(self):
        condition = lambda obj: True
        with self.assertRaises(RuntimeError):
            with self.assertTraitChangesInEventLoop(
                    self.my_class, 'number', condition):
                raise RuntimeError()

    def test_event_loop_until_condition(self):
        obj = EventLoopUser()
        condition = lambda: obj.value == obj.maximum_value
        with self.event_loop_until_condition(condition, timeout=2.0):
            deferred_call(obj.increase_value)

    def test_event_loop_until_condition_timeout(self):
        obj = EventLoopUser(value=10, maximum_value=1)
        condition = lambda: obj.value == obj.maximum_value
        with self.assertRaises(AssertionError):
            with self.event_loop_until_condition(condition, timeout=0.5):
                deferred_call(obj.increase_value)

    def test_event_loop_until_condition_exception(self):
        obj = EventLoopUser()
        condition = lambda: obj.value == obj.maximum_value
        with self.assertRaises(RuntimeError):
            with self.event_loop_until_condition(condition, timeout=1.0):
                raise RuntimeError()

    def test_event_loop_until_trait_change(self):
        with self.assertRaises(AssertionError):
            with self.event_loop_until_traits_change(
                    self.my_class, 'number', timeout=1.0):
                pass

        with reverse_assertion(
                self.assertRaises(AssertionError),
                'Assertion should not be raised'):
            with self.event_loop_until_traits_change(
                    self.my_class, 'number'):
                deferred_call(self._set_trait, 5.0)

    def test_find_qt_widget(self):
        app = self.qt_app
        self.assertIsNone(self.find_qt_widget(app, QtGui.QDockWidget))
        self.assertIsInstance(
            self.find_qt_widget(app, QtGui.QSessionManager),
            QtGui.QSessionManager)

    def test_delete_widget(self):

        class Widget(QtCore.QObject):
            destroyed = QtCore.Signal(bool)

        widget = Widget()

        with self.assertRaises(AssertionError):
            with self.delete_widget(widget, timeout=1.0):
                pass

        with self.delete_widget(widget, timeout=1.0):
            deferred_call(widget.destroyed.emit, True)


class TestGuiTestHelperFunctions(GuiTestAssistant, unittest.TestCase):

    def test_print_qt_widget_tree(self):
        stream = StringIO.StringIO()
        old_stdout = sys.stdout
        try:
            sys.stdout = stream
            print_qt_widget_tree(self.qt_app)
        finally:
            sys.stdout = old_stdout
            stream.close()
        lines = ''.join(stream.buflist).splitlines()
        # basic check we should have four items in the hierarchy.
        self.assertEqual(len(lines), 4)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import unittest

from traits.api import HasTraits, Str

from traits_enaml.testing.enaml_test_assistant import EnamlTestAssistant


class Model(HasTraits):

    txt = Str('foo')


class TestTraitsView(EnamlTestAssistant, unittest.TestCase):

    def setUp(self):

        EnamlTestAssistant.setUp(self)

        enaml_source = """
from enaml.widgets.api import MainWindow
from traits_enaml.widgets.traits_view import TraitsView

enamldef MainView(MainWindow): win:
    attr model

    TraitsView:
        model = win.model
"""

        self.model = Model()

        view, toolkit_view = self.parse_and_create(enaml_source,
                                                   model=self.model)

        self.view = view
        self.traits_view = self.find_enaml_widget(view, "TraitsView")

    def tearDown(self):
        self.traits_view = None
        self.view = None
        self.model = None

        EnamlTestAssistant.tearDown(self)

    def test_disposing_traits_view(self):
        control = self.traits_view.ui.control

        self.view.destroy()
        self.assertEqual(self.traits_view.ui.control, None)

        self.assertEqual(control.parent(), None)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
# load qt from enaml in order to set the setapi before the tests import
# QtCore (see github issue #39).
from enaml import qt
del qt



#---------------------------------------------------------------------------
#
#  Copyright (c) 2012-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import contextlib
import threading

from enaml.application import deferred_call
from enaml.qt.qt_application import QtApplication
from enaml.qt.QtGui import QApplication
from traits.testing.unittest_tools import UnittestTools
from traits.testing.unittest_tools import _TraitsChangeCollector as \
    TraitsChangeCollector

from .event_loop_helper import EventLoopHelper, ConditionTimeoutError


def print_qt_widget_tree(widget, level=0):
    """ Debugging helper to print out the Qt widget tree starting at a
    particular `widget`.

    Parameters
    ----------
    widget: QObject
        The root widget in the tree to print.
    level: int
        The current level in the tree. Used internally for displaying the
        tree level.

    """
    level = level + 4
    if level == 0:
        print
    print ' '*level, widget
    for child in widget.children():
        print_qt_widget_tree(child, level=level)
    if level == 0:
        print


class GuiTestAssistant(UnittestTools):

    def setUp(self):
        qt_app = QApplication.instance()
        if qt_app is None:
            qt_app = QApplication([])
        self.qt_app = qt_app
        enaml_app = QtApplication.instance()
        if enaml_app is None:
            enaml_app = QtApplication()
        self.enaml_app = enaml_app
        self.event_loop_helper = EventLoopHelper(qt_app=self.qt_app)

    def tearDown(self):
        with self.event_loop_with_timeout(repeat=5):
            deferred_call(self.qt_app.closeAllWindows)
        del self.event_loop_helper
        self.enaml_app.destroy()
        del self.enaml_app
        del self.qt_app

    @contextlib.contextmanager
    def event_loop(self, repeat=1):
        """Artificially replicate the event loop by Calling sendPostedEvents
        and processEvents ``repeat`` number of times. If the events to be
        processed place more events in the queue, begin increasing the value
        of ``repeat``, or consider using ``event_loop_until_condition``
        instead.

        Parameters
        ----------
        repeat : int
            Number of times to process events.

        """
        yield
        self.event_loop_helper.event_loop(repeat=repeat)

    @contextlib.contextmanager
    def delete_widget(self, widget, timeout=1.0):
        """Runs the real Qt event loop until the widget provided has been
        deleted.

        Parameters
        ----------
        widget : QObject
            The widget whose deletion will stop the event loop.

        timeout : float
            Number of seconds to run the event loop in the case that the
            widget is not deleted.

        """
        try:
            with self.event_loop_helper.delete_widget(widget, timeout=timeout):
                yield
        except ConditionTimeoutError:
            self.fail('Could not destroy widget before timeout: {!r}'.format(
                widget))

    @contextlib.contextmanager
    def event_loop_until_condition(self, condition, timeout=10.0):
        """Runs the real Qt event loop until the provided condition evaluates
        to True.

        This should not be used to wait for widget deletion. Use
        delete_widget() instead.

        Parameters
        ----------
        condition : callable
            A callable to determine if the stop criteria have been met. This
            should accept no arguments.

        timeout : float
            Number of seconds to run the event loop in the case that the
            condition is not satisfied.

        """
        try:
            yield
            self.event_loop_helper.event_loop_until_condition(
                condition, timeout=timeout)
        except ConditionTimeoutError:
            self.fail('Timed out waiting for condition')

    @contextlib.contextmanager
    def assertTraitChangesInEventLoop(self, obj, trait, condition, count=1,
                                      timeout=10.0):
        """Runs the real Qt event loop, collecting trait change events until
        the provided condition evaluates to True.

        Parameters
        ----------
        obj : HasTraits
            The HasTraits instance whose trait will change.

        trait : str
            The extended trait name of trait changes to listen too.

        condition : callable
            A callable to determine if the stop criteria have been met. This
            should accept no arguments.

        count : int
            The expected number of times the event should be fired. The default
            is to expect one event.

        timeout : float
            Number of seconds to run the event loop in the case that the trait
            change does not occur.

        """
        condition_ = lambda: condition(obj)
        collector = TraitsChangeCollector(obj=obj, trait=trait)

        collector.start_collecting()
        try:
            try:
                yield collector
                self.event_loop_helper.event_loop_until_condition(
                    condition_, timeout=timeout)
            except ConditionTimeoutError:
                actual_event_count = collector.event_count
                msg = ("Expected {} event on {} to be fired at least {} "
                       "times, but the event was only fired {} times "
                       "before timeout ({} seconds).")
                msg = msg.format(
                    trait, obj, count, actual_event_count, timeout)
                self.fail(msg)
        finally:
            collector.stop_collecting()

    @contextlib.contextmanager
    def event_loop_until_traits_change(self, traits_object, *traits, **kw):
        """Run the real application event loop until a change notification for
        all of the specified traits is received.

        Paramaters
        ----------
        traits_object: HasTraits instance
            The object on which to listen for a trait events
        traits: one or more str
            The names of the traits to listen to for events
        timeout: float, optional, keyword only
            Number of seconds to run the event loop in the case that the trait
            change does not occur. Default value is 10.0.

        """
        timeout = kw.pop('timeout', 10.0)
        condition = threading.Event()

        traits = set(traits)
        recorded_changes = set()

        def set_event(trait):
            recorded_changes.add(trait)
            if recorded_changes == traits:
                condition.set()

        handlers = {}
        for trait in traits:
            handlers[trait] = lambda: set_event(trait)

        for trait, handler in handlers.iteritems():
            traits_object.on_trait_change(handler, trait)
        try:
            with self.event_loop_until_condition(
                    condition=condition.is_set, timeout=timeout):
                yield
        finally:
            for trait, handler in handlers.iteritems():
                traits_object.on_trait_change(handler, trait, remove=True)

    def find_qt_widget(self, start, type_, test=None):
        """Recursively walks the Qt widget tree from Qt widget `start` until it
        finds a widget of type `type_` (a QWidget subclass) that
        satisfies the provided `test` method.

        Parameters
        ----------
        start: QWidget
            The widget from which to start walking the tree
        type_: type
            A subclass of QWidget to use for an initial type filter while
            walking the tree
        test: callable
            A filter function that takes one argument (the current widget being
            evaluated) and returns either True or False to determine if the
            widget matches the required criteria.

        """
        if test is None:
            test = lambda widget: True
        if isinstance(start, type_):
            if test(start):
                return start
        for child in start.children():
            widget = self.find_qt_widget(child, type_, test=test)
            if widget:
                return widget
        return None

    @contextlib.contextmanager
    def event_loop_with_timeout(self, repeat=2, timeout=10.0):
        """Helper context manager to send all posted events to the event queue
        and wait for them to be processed.

        This differs from the `event_loop()` context manager in that it
        starts the real event loop rather than emulating it with
        `QApplication.processEvents()`

        Parameters
        ----------
        repeat : int
            Number of times to process events. Default is 2.
        timeout: float, optional, keyword only
            Number of seconds to run the event loop in the case that the trait
            change does not occur. Default value is 10.0.

        """
        yield
        self.event_loop_helper.event_loop_with_timeout(
            repeat=repeat, timeout=timeout)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
""" An amalgamation of utilities used throughout the Traits Enaml framework.

"""
from collections import defaultdict


class abstractclassmethod(classmethod):
    """ A backport of the Python 3's abc.abstractclassmethod.

    """
    __isabstractmethod__ = True

    def __init__(self, func):
        func.__isabstractmethod__ = True
        super(abstractclassmethod, self).__init__(func)


class LoopbackContext(object):
    """ A context manager generated by LoopbackGuard.

    Instances of this class manage acquiring and releasing the lock
    items for instances of LoopbackGuard.

    """
    __slots__ = ('_guard', '_items')

    def __init__(self, guard, items):
        """ Initialize a LoopbackContext

        Parameters
        ----------
        guard : LoopbackGuard
            The loopback guard instance for which we will acquire the
            lock for the items.

        items : iterable
            An iterable items which will be passed to the 'acquire'
            method on the loopback guard.

        """
        self._guard = guard
        self._items = tuple(items)

    def __enter__(self):
        """ Acquire the guard lock on the lock items.

        """
        self._guard.acquire(self._items)

    def __exit__(self, exc_type, exc_value, traceback):
        """ Release the guard lock on the lock items.

        """
        self._guard.release(self._items)


class LoopbackGuard(object):
    """ A guard object to protect against feedback loops.

    Instances of this class are used by objects to protect against
    loopback conditions while updating attributes. Instances of this
    class are callable and return a guarding context manager for the
    provided lock items. The guard can be tested for a locked item
    using the `in` keyword.

    """
    __slots__ = ('locked_items',)

    def __init__(self):
        """ Initialize a loopback guard.

        """
        self.locked_items = None

    def __call__(self, *items):
        """ Return a context manager which will guard the given items.

        Parameters
        ----------
        items
            The items for which to acquire the guard from within the
            returned context manager. These items must be hashable.

        Returns
        -------
        result : LoopbackContext
            A context manager which will acquire the guard for the
            provided items.

        """
        return LoopbackContext(self, items)

    def __contains__(self, item):
        """ Returns whether or not the given item is currently guarded.

        Parameters
        ----------
        item : object
            The item to check for guarded state.

        Returns
        -------
        result : bool
            True if the item is currently guarded, False otherwise.

        """
        locked_items = self.locked_items
        if locked_items is not None:
            return item in locked_items
        return False

    def acquire(self, items):
        """ Acquire the guard for the given items.

        This method is normally called by the LoopbackContext returned
        by calling this instance. User code should not typically call
        this method directly. It is safe to call this method multiple
        times for and item, provided it is paired with the same number
        of calls to release(...). The guard will be released when the
        acquired count on the item reaches zeros.

        Parameters
        ----------
        items : iterable
            An iterable of objects for which to acquire the guard. The
            items must be hashable.

        """
        locked_items = self.locked_items
        if locked_items is None:
            locked_items = self.locked_items = defaultdict(int)
        for item in items:
            locked_items[item] += 1

    def release(self, items):
        """ Release the guard for the given lock items.

        This method is normally called by the LoopbackContext returned
        by calling this instance. User code should not normally call
        this method directly. It is safe to call this method multiple
        times for and item, provided it is paired with the same number
        of calls to acquire(...). The guard will be released when the
        acquired count on the item reaches zeros.

        Parameters
        ----------
        items : iterable
            An iterable of objects for which to release the guard. The
            items must be hashable.

        """
        locked_items = self.locked_items
        if locked_items is not None:
            for item in items:
                locked_items[item] -= 1
                if locked_items[item] <= 0:
                    del locked_items[item]
            if not locked_items:
                self.locked_items = None



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
from traits.api import HasTraits, Disallow, TraitListObject, TraitDictObject

from enaml.core.standard_tracer import StandardTracer, SubscriptionObserver


class TraitsObserver(SubscriptionObserver):
    """ An observer object which manages a tracer subscription.

    Subclassed to allow weak-referencing.

    """
    __slots__ = ('__weakref__',)


class TraitsTracer(StandardTracer):
    """ A CodeTracer for tracing expressions which use Traits.

    This tracer maintains a running set of `traced_traits` which are the
    (obj, name) pairs of traits items discovered during tracing.

    """
    __slots__ = 'traced_traits'

    def __init__(self, owner, name):
        """ Initialize a TraitsTracer.

        """
        super(TraitsTracer, self).__init__(owner, name)
        self.traced_traits = set()

    #--------------------------------------------------------------------------
    # Private API
    #--------------------------------------------------------------------------
    def _trace_trait(self, obj, name):
        """ Add the trait object and name pair to the traced items.

        Parameters
        ----------
        obj : HasTraits
            The traits object owning the attribute.

        name : str
            The trait name to for which to bind a handler.

        """
        # Traits will happily force create a trait for things which aren't
        # actually traits. This tries to avoid most of that when possible.
        trait = obj.trait(name)
        if trait is not None and trait.trait_type is not Disallow:
            self.traced_traits.add((obj, name))
            # Check for collections.
            if trait.handler is not None and trait.handler.has_items:
                self.traced_traits.add((obj, '{}_items'.format(name)))

    #--------------------------------------------------------------------------
    # AbstractScopeListener Interface
    #--------------------------------------------------------------------------
    def dynamic_load(self, obj, attr, value):
        """ Called when an object attribute is dynamically loaded.

        This will trace the object if it is a HasTraits instance.
        See also: `AbstractScopeListener.dynamic_load`.

        """
        super(TraitsTracer, self).dynamic_load(obj, attr, value)
        if isinstance(obj, HasTraits):
            self._trace_trait(obj, attr)

    #--------------------------------------------------------------------------
    # CodeTracer Interface
    #--------------------------------------------------------------------------
    def load_attr(self, obj, attr):
        """ Called before the LOAD_ATTR opcode is executed.

        This will trace the object if it is a HasTraits instance.
        See also: `CodeTracer.dynamic_load`.

        """
        super(TraitsTracer, self).load_attr(obj, attr)
        if isinstance(obj, HasTraits):
            self._trace_trait(obj, attr)

    def call_function(self, func, argtuple, argspec):
        """ Called before the CALL_FUNCTION opcode is executed.

        This will trace the func is the builtin `getattr` and the object
        is a HasTraits instance. See also: `CodeTracer.call_function`

        """
        super(TraitsTracer, self).call_function(func, argtuple, argspec)
        nargs = argspec & 0xFF
        nkwargs = (argspec >> 8) & 0xFF
        if (func is getattr and (nargs == 2 or nargs == 3) and nkwargs == 0):
            obj, attr = argtuple[0], argtuple[1]
            if isinstance(obj, HasTraits) and isinstance(attr, basestring):
                self._trace_trait(obj, attr)

    def binary_subscr(self, obj, idx):
        """ Called before the BINARY_SUBSCR opcode is executed.

        This will trace the object if it is a `TraitListObject` or a
        `TraitDictObject`. See also: `CodeTracer.get_iter`.

        """
        super(TraitsTracer, self).binary_subscr(obj, idx)
        if isinstance(obj, (TraitListObject, TraitDictObject)):
            traits_obj = obj.object()
            if traits_obj is not None:
                if obj.name_items:
                    self._trace_trait(traits_obj, obj.name_items)

    def get_iter(self, obj):
        """ Called before the GET_ITER opcode is executed.

        This will trace the object if it is a `TraitListObject`
        See also: `CodeTracer.get_iter`.

        """
        super(TraitsTracer, self).get_iter(obj)
        if isinstance(obj, TraitListObject):
            traits_obj = obj.object()
            if traits_obj is not None:
                if obj.name_items:
                    self._trace_trait(traits_obj, obj.name_items)

    #--------------------------------------------------------------------------
    # StandardTracer Interface
    #--------------------------------------------------------------------------
    def finalize(self):
        """ Finalize the tracing process.

        This method will discard the old observer and attach a new
        observer to the traced dependencies.

        """
        owner = self.owner
        name = self.name
        key = '_[%s|trace]' % name
        storage = owner._d_storage

        # invalidate the old observer so that it can be collected
        old_observer = storage.get(key)
        if old_observer is not None:
            old_observer.ref = None

        # create a new observer and subscribe it to the dependencies
        if self.items or self.traced_traits:
            observer = TraitsObserver(owner, name)
            storage[key] = observer
            for obj, d_name in self.items:
                obj.observe(d_name, observer)
            for obj, d_name in self.traced_traits:
                obj.on_trait_change(observer.__call__, d_name)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import unittest

from enable.api import Component
from traits.api import HasTraits, Instance

from traits_enaml.testing.enaml_test_assistant import EnamlTestAssistant


class Model(HasTraits):

    component = Instance(Component)


class EnableCanvasTestCase(EnamlTestAssistant, unittest.TestCase):

    def setUp(self):

        EnamlTestAssistant.setUp(self)

        enaml_source = """
from enaml.widgets.api import MainWindow
from traits_enaml.widgets.enable_canvas import EnableCanvas

enamldef MainView(MainWindow):
    attr model

    EnableCanvas:
        name = 'canvas'
        component << model.component
"""
        self.component = Component()
        self.model = Model(component=self.component)

        view, toolkit_view = self.parse_and_create(
            enaml_source, model=self.model
        )

        self.view = view

    def tearDown(self):
        self.component = None
        self.view = None
        self.model = None

        EnamlTestAssistant.tearDown(self)

    def test_using_enable_canvas_widget(self):

        canvas = self.view.find('canvas')

        with self.assertAtomChanges(canvas, 'component'):
            self.model.component = Component()

        canvas = None

    def test_enable_canvas_proxy(self):

        from enaml.qt.qt_raw_widget import QtRawWidget

        canvas = self.view.find('canvas')
        # Check that the proxy is strictly a QtRawWidget (not a subclass).
        self.assertEqual(type(canvas.proxy), QtRawWidget)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2012-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import contextlib
import threading

from enaml.application import deferred_call
from enaml.qt import QtCore, QtGui
from traits.api import HasStrictTraits, Instance


class ConditionTimeoutError(RuntimeError):
    pass


class EventLoopHelper(HasStrictTraits):

    qt_app = Instance(QtGui.QApplication)

    def event_loop_with_timeout(self, repeat=2, timeout=10.0):
        """Helper function to send all posted events to the event queue and
        wait for them to be processed.  This runs the real event loop and
        does not emulate it with QApplication.processEvents.

        Parameters
        ----------
        repeat : int
            Number of times to process events. Default is 2.
        timeout: float, optional, keyword only
            Number of seconds to run the event loop. Default value is 10.0.

        """
        def repeat_loop(condition, repeat):
            # We sendPostedEvents to ensure that enaml events are processed
            self.qt_app.sendPostedEvents()
            repeat = repeat - 1
            if repeat <= 0:
                deferred_call(condition.set)
            else:
                deferred_call(repeat_loop, condition=condition, repeat=repeat)

        condition = threading.Event()
        deferred_call(repeat_loop, repeat=repeat, condition=condition)
        self.event_loop_until_condition(
            condition=condition.is_set, timeout=timeout)

    def event_loop(self, repeat=1):
        """Emulates an event loop `repeat` times with
        QApplication.processEvents.

        Parameters
        ----------
        repeat : int
            Number of times to process events. Default is 1.

        """
        for i in range(repeat):
            self.qt_app.sendPostedEvents()
            self.qt_app.processEvents()

    def event_loop_until_condition(self, condition, timeout=10.0):
        """Runs the real Qt event loop until the provided condition evaluates
        to True.

        Raises ConditionTimeoutError if the timeout occurs before the condition
        is satisfied.

        Parameters
        ----------
        condition : callable
            A callable to determine if the stop criteria have been met. This
            should accept no arguments.

        timeout : float
            Number of seconds to run the event loop in the case that the trait
            change does not occur.

        """
        def handler():
            if condition():
                self.qt_app.quit()
        condition_timer = QtCore.QTimer()
        condition_timer.setInterval(50)
        condition_timer.timeout.connect(handler)
        timeout_timer = QtCore.QTimer()
        timeout_timer.setSingleShot(True)
        timeout_timer.setInterval(timeout * 1000)
        timeout_timer.timeout.connect(self.qt_app.quit)
        timeout_timer.start()
        condition_timer.start()
        try:
            self.qt_app.exec_()
            if not timeout_timer.isActive():
                # We exited the event loop on timeout
                raise ConditionTimeoutError('Timed out waiting for condition')
        finally:
            timeout_timer.stop()
            condition_timer.stop()

    @contextlib.contextmanager
    def delete_widget(self, widget, timeout=1.0):
        """Runs the real Qt event loop until the widget provided has been
        deleted.  Raises ConditionTimeoutError on timeout.

        Parameters
        ----------
        widget : QObject
            The widget whose deletion will stop the event loop.

        timeout : float
            Number of seconds to run the event loop in the case that the
            widget is not deleted.

        """
        timer = QtCore.QTimer()
        timer.setSingleShot(True)
        timer.setInterval(timeout * 1000)
        timer.timeout.connect(self.qt_app.quit)
        widget.destroyed.connect(self.qt_app.quit)
        yield
        timer.start()
        self.qt_app.exec_()
        if not timer.isActive():
            # We exited the event loop on timeout
            raise ConditionTimeoutError(
                'Could not destroy widget before timeout: {!r}'.format(widget))



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
from traits.etsconfig.api import ETSConfig

import enaml

from .trait_operators import TRAIT_OPERATORS

if ETSConfig.toolkit not in ['', 'qt4']:
    raise ValueError('traits-enaml does not support WX')

ETSConfig.toolkit = 'qt4'


def imports():
    return enaml.imports(operators=TRAIT_OPERATORS)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import unittest

from traits.api import HasTraits, Str

from traits_enaml.testing.enaml_test_assistant import EnamlTestAssistant
from traits_enaml.widgets.traits_view import TraitsView


class Model(HasTraits):

    txt = Str('foo')


class TestTraitsViewDynamic(EnamlTestAssistant, unittest.TestCase):

    def setUp(self):

        EnamlTestAssistant.setUp(self)

        enaml_source = """
from enaml.widgets.api import MainWindow
from enaml.core.api import Include

enamldef MainView(MainWindow): win:
    attr traits_view


    Include:
        objects << [traits_view] if traits_view is not None else []
"""

        view, toolkit_view = self.parse_and_create(enaml_source)
        self.view = view

    def tearDown(self):
        self.view = None
        self.model = None

        EnamlTestAssistant.tearDown(self)

    def test_add_traits_view(self):
        traits_view = TraitsView(model=Model())
        with self.event_loop():
            self.view.show()
        with self.event_loop():
            self.view.traits_view = traits_view
        self.assertEqual(traits_view.ui.rebuild.__name__, 'ui_subpanel')


if __name__ == "__main__":
    unittest.main()



#------------------------------------------------------------------------------
#  Copyright (c) 2013, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from atom.api import Bool, Typed, observe, set_default
from enaml.core.declarative import d_

from traits.api import Instance, HasStrictTraits
from traitsui.api import View, Item
from tvtk.pyface.scene_editor import SceneEditor
from tvtk.pyface.scene_model import SceneModel

from .traits_view import TraitsView


class MayaviModel(HasStrictTraits):
    scene = Instance(SceneModel, args=())
    view = View(
        Item('scene', editor=SceneEditor(), resizable=True, show_label=False),
        resizable=True)


class MayaviCanvas(TraitsView):
    """ A traits view widget that displays a mayavi scene.

    :Attributes:
        **scene** = *d_(Typed(SceneModel))*
            The mayavi scene model to be displayed.
        **show_toolbar** = *d_(Bool(True))*
            If True, show the Mayavi toolbar.

    """
    #: The mayavi scene model to be displayed.
    scene = d_(Typed(SceneModel))

    #: If True, show the Mayavi toolbar.
    show_toolbar = d_(Bool(True))

    #: The readonly instance of the model used for the traits_view.
    model = d_(Typed(MayaviModel), writable=False)

    #: The readonly instance of the view.
    view = d_(Typed(View), writable=False)

    #: Mayavi canvas expands freely in width and height by default.
    hug_width = set_default('ignore')
    hug_height = set_default('ignore')

    def create_widget(self, parent):
        control = super(MayaviCanvas, self).create_widget(parent)
        self.show_toolbar_changed({'value': self.show_toolbar})
        return control

    def _default_model(self):
        return MayaviModel(scene=self.scene)

    def _default_view(self):
        return self.model.trait_view()

    @observe('scene')
    def scene_changed(self, change):
        self.model.scene = change['value']

    @observe('show_toolbar')
    def show_toolbar_changed(self, change):
        ui = self.ui
        if ui is not None and ui.control is not None:
            editor = ui.get_editors('scene')[0]
            editor._scene._tool_bar.setVisible(change['value'])



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
"""
Unit tests for the LoopbackGuard class and LoopbackContext context manager.

"""
import unittest

from traits.api import HasTraits, Instance, List, on_trait_change
from traits_enaml.utils import LoopbackGuard, LoopbackContext


class TestLoopbackGuard(unittest.TestCase):
    """
    Unit tests that exercise the LoopbackGuard class directly.

    """
    def test_acquire_release_balanced(self):
        # Check that we can acquire and release the loopback guard
        # multiple times.

        to_add = {'item 0': 3, 'item 1': 4, 'item 2': 2}
        guard = LoopbackGuard()

        # Acquire the guard for the items in to_add.
        for obj, times in to_add.items():
            for _ in range(times):
                guard.acquire([obj])

        self.assertDictEqual(guard.locked_items, to_add)

        # Release the guard.
        for obj, times in to_add.items():
            for _ in range(times):
                guard.release([obj])

        self.assertIsNone(guard.locked_items)


class TestLoopbackContext(unittest.TestCase):
    """
    Unit tests that exercise the LoopbackGuard class through
    the LoopbackContext context manager.

    """

    def _check_loopback_context_manager(self, cm, guard, items):
        # Check that we can add items to a loopback context manager, and that
        # the items are properly released when the context is exited.

        with cm:
            self.assertItemsEqual(guard.locked_items, items)
            for item in items:
                self.assertIn(item, guard)

        self.assertIsNone(guard.locked_items)
        for item in items:
            self.assertNotIn(item, guard)

    def test_loopback_context_direct(self):
        # Instantiate a context manager directly.
        items = ['a', 'b', 'c']
        guard = LoopbackGuard()
        cm = LoopbackContext(guard, items)
        self._check_loopback_context_manager(cm, guard, items)

    def test_loopback_context_call(self):
        # Instantiate a context manager by calling the loopback guard with the
        # items to be locked.
        items = ['a', 'b', 'c']
        guard = LoopbackGuard()
        cm = guard(*items)
        self._check_loopback_context_manager(cm, guard, items)


class Dummy(object):
    pass


class E(HasTraits):

    a = Instance(Dummy, ())

    b = Instance(Dummy, ())

    # List of traits that have updated.
    updates = List

    # Loopback guard to protect `a` from cyclic updates.
    guard = Instance(LoopbackGuard, ())

    @on_trait_change('a')
    def _a_updated(self, new):
        with self.guard(self.a):
            self.updates.append('b')
            self.b = new

    @on_trait_change('b')
    def _b_updated(self, new):
        if self.a not in self.guard:
            self.updates.append('a')
            self.a = new


class TestLoopbackGuardUseCase(unittest.TestCase):
    """
    Test suite for the common LoopbackGuard use case.

    """
    def test_loopback_prevents_cyclic_updates(self):
        # Check that the loopback guard prevents cyclic updates in `a`.
        e = E()
        e.a = Dummy()
        self.assertEqual(e.updates, ['b'])

    def test_normal_cyclic_updates(self):
        # Test that the loopback guard does not stand in the way of the
        # normal traits behavior when `b` is updated.
        e = E()
        e.b = Dummy()  # The loopback guard does not protect `b`.
        self.assertEqual(e.updates, ['a', 'b'])



#------------------------------------------------------------------------------
#  Copyright (c) 2013, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from atom.api import Typed, observe, set_default
from enable.api import Window, Component

from enaml.widgets.api import RawWidget
from enaml.core.declarative import d_


class EnableCanvas(RawWidget):
    """ A widget that displays an enable component

    :Attributes:
        **component** = *d_(Typed(Component))*
            The enable component to be displayed

    """
    #: The enable component to be displayed
    component = d_(Typed(Component))

    #: Internal storage for the enable window
    _window = Typed(Window)

    #: Enable canvas' expand freely in width and height by default
    hug_width = set_default('ignore')
    hug_height = set_default('ignore')

    def create_widget(self, parent):
        if self.component is not None:
            self._window = Window(parent, component=self.component,
                                  bgcolor=self.component.bgcolor)
            enable_widget = self._window.control
        else:
            self._window = None
            enable_widget = None

        return enable_widget

    @observe('component')
    def component_changed(self, new):
        if self._window is not None:
            self._window.component = new['value']



#
# (C) Copyright 2013 Enthought, Inc., Austin, TX
# All right reserved.
#
# This file is open source software distributed according to the terms in
# LICENSE.txt
#

from collections import namedtuple
from datetime import date, time
from string import capwords

from traits.api import (BaseInstance, Bool, Button, Enum, Event, Float, Int,
                        Range, Str)
import traits_enaml

with traits_enaml.imports():
    from enaml.widgets.api import Label
    from traits_enaml.widgets.auto_editors import (
        AutoView, AutoWindow, BoolEditor, ButtonEditor,
        DateEditor, EnumEditor, FloatEditor, FloatRangeEditor,
        IntEditor, IntRangeEditor, StrEditor, TimeEditor,
        DefaultEditor)

TraitDesc = namedtuple('TraitDesc', 'name trait_type label tooltip editor')

# Dictionary from trait_type -> enaml component factories.
TRAIT2ENAML = {
    Bool: lambda trait_type: BoolEditor,
    Button: lambda trait_type: ButtonEditor,
    Enum: lambda trait_type: EnumEditor,
    Float: lambda trait_type: FloatEditor,
    Int: lambda trait_type: IntEditor,
    Str: lambda trait_type: StrEditor,
    Range: lambda trait_type: _range_editor_factory(trait_type),
    BaseInstance: lambda trait_type: _time_editor_factory(trait_type)}


def auto_view(model):
    """ Generate a view directly from a `HasTraits` instance.
    """
    descriptions = _model_traits(model)
    labels = [Label(text=desc.label) for desc in descriptions]
    editors = [_get_editor(model, desc) for desc in descriptions]
    objects = [widget for pair in zip(labels, editors) for widget in pair]
    return AutoView(objects=objects)


def auto_window(model):
    """ Generate a window directly from a `HasTraits` instance.
    """
    return AutoWindow(view=auto_view(model))


def _get_editor(model, trait_desc):
    kwargs = {'model': model, 'trait_desc': trait_desc}
    if trait_desc.tooltip:
        kwargs['tool_tip'] = trait_desc.tooltip

    if trait_desc.editor:
        return trait_desc.editor(**kwargs)
    else:
        trait_type = trait_desc.trait_type
        for key, factory in TRAIT2ENAML.iteritems():
            if isinstance(trait_type, key):
                break
        else:
            factory = lambda trait: DefaultEditor
        editor = factory(trait_type)
        return editor(**kwargs)


def _range_editor_factory(trait_type):
    low, high = trait_type._low, trait_type._high
    if isinstance(low, float) and isinstance(high, float):
        return FloatRangeEditor
    elif isinstance(low, int) and isinstance(high, int):
        return IntRangeEditor
    else:
        return DefaultEditor


def _time_editor_factory(trait_type):
    klass = trait_type.klass
    if klass is time:
        return TimeEditor
    elif klass is date:
        return DateEditor
    else:
        return DefaultEditor


def _model_traits(model):
    traits = []
    names = [
        name for name in model.class_trait_names()
        if not name.startswith('_')]
    for name in names:
        trait = model.trait(name)
        if type(trait.trait_type) is Event:
            continue
        label = trait.label or capwords(name.replace('_', ' '))
        tooltip = trait.tooltip
        editor = trait.enaml_editor
        desc = TraitDesc(name, trait.trait_type, label, tooltip, editor)
        traits.append(desc)
    return traits



#
# (C) Copyright 2013 Enthought, Inc., Austin, TX
# All right reserved.
#
# This file is open source software distributed according to the terms in
# LICENSE.txt
#
from traits.api import HasTraits
from traitsui.api import View

from atom.api import Typed, set_default

from enaml.core.declarative import d_
from enaml.widgets.raw_widget import RawWidget


class TraitsView(RawWidget):
    """ A widget which wraps a TraitsUI View on an object.

    :Attributes:
        **model** = `d_(Typed(HasTraits))`
            The HasTraits instance that we are using.
        **view** = `d_(Typed(View))`
            The View instance that we are using.
        **ui** = `Typed(HasTraits)`
            A reference to the TraitsUI UI object.
        **hug_width** = `set_default('weak')`
            TraitsViews hug their contents' width weakly by default.

    """

    #: The HasTraits instance that we are using.
    model = d_(Typed(HasTraits))

    #: The View instance that we are using.
    view = d_(Typed(View))

    #: A reference to the TraitsUI UI object.
    ui = Typed(HasTraits)

    #: TraitsViews hug their contents' width weakly by default.
    hug_width = set_default('weak')

    def create_widget(self, parent):
        self.ui = self.model.edit_traits(
            self.view, parent=parent, kind='subpanel')
        self.ui.control.setParent(parent)
        return self.ui.control

    def destroy_widget(self):
        control = self.ui.control
        if control is not None:
            control.setParent(None)
            self.ui.dispose()

    def destroy(self):
        """ A reimplemented destructor.

        This destructor disposes the TraitsUI object before proceeding with
        the regular Enaml destruction.

        """
        self.destroy_widget()
        super(TraitsView, self).destroy()



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import unittest

from traits.api import (
    Event, Float, HasTraits, Str, List, Dict, Set, Property)
from traits_enaml.testing.enaml_test_assistant import EnamlTestAssistant


class TraitModel(HasTraits):
    value = Float
    value_delegate = Str()
    value_subscribe = Str()
    value_update = Str()
    value_simple = Str('simple_text')
    value_notify = Event()
    list_values = List(Str)
    dict_values = Dict(Str, Str)
    set_values = Set(Str)
    property_value = Property(depends_on='value')
    typed_property_value = Property(Float, depends_on='value')
    collection_property_value = Property(List, depends_on='value')

    def _get_property_value(self):
        return self.value

    def _get_typed_property_value(self):
        return self.value

    def _get_collection_property_value(self):
        value = self.value
        return [value, value * 10]


class TraitOperatorsTestCase(EnamlTestAssistant, unittest.TestCase):

    def setUp(self):

        EnamlTestAssistant.setUp(self)

        enaml_source = """
from enaml.widgets.api import MainWindow, Field

enamldef MainView(MainWindow):
    attr model
    Field:
        name = 'test_op_delegate'
        text := model.value_delegate
    Field:
        name = 'test_op_subscribe'
        text << model.value_subscribe
    Field:
        name = 'test_op_update'
        text >> model.value_update
    Field:
        name = 'test_op_simple'
        text = model.value_simple
    Field:
        name = 'test_op_notify'
        text ::
            model.value_notify = True
    Field:
        name = 'test_list_subscribe'
        text << str(model.list_values)
    Field:
        name = 'test_dict_subscribe'
        text << str(model.dict_values)
    Field:
        name = 'test_set_subscribe'
        text << str(model.set_values)
    Field:
        name = 'test_property_subscribe'
        text << str(model.property_value)
    Field:
        name = 'test_typed_property_subscribe'
        text << str(model.typed_property_value)
    Field:
        name = 'test_collection_property_subscribe'
        text << str(model.collection_property_value)
"""
        self.model = TraitModel()
        view, toolkit_view = self.parse_and_create(
            enaml_source, model=self.model
        )

        self.view = view

    def tearDown(self):
        self.view = None
        self.model = None
        EnamlTestAssistant.tearDown(self)

    def test_op_delegate(self):

        enaml_widget = self.view.find('test_op_delegate')

        with self.assertTraitChanges(self.model, 'value_delegate'):
            enaml_widget.text = 'new_value'

        self.assertEquals(self.model.value_delegate, 'new_value')

        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.value_delegate = 'updated_trait'

        self.assertEquals(enaml_widget.text, 'updated_trait')

    def test_op_subscribe(self):

        enaml_widget = self.view.find('test_op_subscribe')

        with self.assertTraitDoesNotChange(self.model, 'value_subscribe'):
            enaml_widget.text = 'new_value'

        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.value_subscribe = 'updated_trait'

        self.assertEquals(enaml_widget.text, 'updated_trait')

    def test_op_update(self):

        enaml_widget = self.view.find('test_op_update')

        with self.assertTraitChanges(self.model, 'value_update'):
            enaml_widget.text = 'new_value'

        self.assertEquals(self.model.value_update, 'new_value')

        with self.assertAtomDoesNotChange(enaml_widget, 'text'):
            self.model.value_subscribe = 'updated_trait'

    def test_op_simple(self):

        enaml_widget = self.view.find('test_op_simple')

        self.assertEquals(self.model.value_simple, enaml_widget.text)
        self.assertEquals('simple_text', enaml_widget.text)

        with self.assertTraitDoesNotChange(self.model, 'value_simple'):
            enaml_widget.text = 'new_value'

        with self.assertAtomDoesNotChange(enaml_widget, 'text'):
            self.model.value_simple = 'updated_trait'

    def test_op_notify(self):

        enaml_widget = self.view.find('test_op_notify')

        with self.assertTraitChanges(self.model, 'value_notify'):
            enaml_widget.text = 'changing text'

        # Updating the text with the same value does not trigger the event
        with self.assertTraitDoesNotChange(self.model, 'value_notify'):
            enaml_widget.text = 'changing text'

        with self.assertTraitChanges(self.model, 'value_notify', count=1):
            enaml_widget.text = 'new text'

    def test_list_subscribe(self):

        enaml_widget = self.view.find('test_list_subscribe')

        with self.assertTraitDoesNotChange(self.model, 'list_values'):
            enaml_widget.text = 'new_value'

        # check on replace
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.list_values = ['1']
            self.assertEquals(enaml_widget.text, "['1']")

        # check on append
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.list_values.append('2')
            self.assertEquals(enaml_widget.text, "['1', '2']")

        # check on remove
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.list_values.remove('1')
            self.assertEquals(enaml_widget.text, "['2']")

    def test_dict_subscribe(self):

        enaml_widget = self.view.find('test_dict_subscribe')

        with self.assertTraitDoesNotChange(self.model, 'dict_values'):
            enaml_widget.text = 'new_value'

        # check on replace
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.dict_values = {'one': '1'}
            self.assertEquals(enaml_widget.text, "{'one': '1'}")

        # check on append
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.dict_values['two'] = '2'
            self.assertEquals(
                enaml_widget.text, str({'one': '1', 'two': '2'}))

        # check on remove
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            del self.model.dict_values['one']
            self.assertEquals(enaml_widget.text, "{'two': '2'}")

    def test_set_subscribe(self):

        enaml_widget = self.view.find('test_set_subscribe')

        with self.assertTraitDoesNotChange(self.model, 'set_values'):
            enaml_widget.text = 'new_value'

        # check on replace
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.set_values = {'1'}
            self.assertEquals(enaml_widget.text, "TraitSetObject(['1'])")

        # check on append
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.set_values.add('2')
            self.assertEquals(enaml_widget.text, "TraitSetObject(['1', '2'])")

        # check on remove
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.set_values.remove('1')
            self.assertEquals(enaml_widget.text, "TraitSetObject(['2'])")

    def test_property_subscribe(self):

        enaml_widget = self.view.find('test_property_subscribe')

        with self.assertTraitDoesNotChange(self.model, 'property_value'):
            enaml_widget.text = 'new_value'

        # check on replace
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.value = 3.4
            self.assertEquals(enaml_widget.text, u"3.4")

    def test_typed_property_subscribe(self):

        enaml_widget = self.view.find('test_typed_property_subscribe')

        with self.assertTraitDoesNotChange(
                self.model, 'typed_property_value'):
            enaml_widget.text = 'new_value'

        # check on replace
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.value = 4.5
            self.assertEquals(enaml_widget.text, u"4.5")

    def test_collection_property_subscribe(self):

        enaml_widget = self.view.find('test_collection_property_subscribe')

        with self.assertTraitDoesNotChange(
                self.model, 'collection_property_value'):
            enaml_widget.text = 'new_value'

        # check on replace
        with self.assertAtomChanges(enaml_widget, 'text', count=1):
            self.model.value = 4.5
            self.assertEquals(enaml_widget.text, u"[4.5, 45.0]")



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------

from setuptools import setup, find_packages


setup(
    name='traits-enaml',
    version='0.2.1',
    author='Enthought, Inc',
    author_email='info@enthought.com',
    url='https://github.com/enthought/traits-enaml',
    description='Utilities for interoperation between Traits and Enaml',
    long_description=open('README.rst').read(),
    packages=find_packages(),
    package_data={'traits_enaml.testing.tests': ['*.enaml'],
                  'traits_enaml.widgets': ['*.enaml']},
    requires=[
    ],
)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
import unittest
import datetime

import traits_enaml
from traits.api import (
    Bool, Button, Date, Enum, Float, Int, HasTraits, List, Range, Str, Time,
    Tuple)
from traits_enaml.testing.enaml_test_assistant import EnamlTestAssistant
from traits_enaml.widgets.auto_view import auto_window, auto_view

with traits_enaml.imports():
    from traits_enaml.widgets.auto_view import DefaultEditor


class AllTypes(HasTraits):
    """ A simple class with all kinds of traits

    """
    boolean_value = Bool(True, label="Custom Bool Label:")
    button_value = Button("I'm a button!")
    int_value = Int(42, tooltip="You can add a tooltip as well.")
    float_value = Float(3.141592)
    enum_value = Enum("foo", "bar", "baz", "qux")
    int_range_value = Range(low=0, high=10)
    float_range_value = Range(low=0.0, high=1.0)
    list_value = List([0, 1, 2])
    str_value = Str("Word")
    date_value = Date(datetime.date.today())
    time_value = Time(datetime.time())
    range_value = Range(low=0, high=100,
                        label="Traits Range Editor:",
                        enaml_editor=DefaultEditor)

    _notifications = List(Tuple)


class TestAutoView(EnamlTestAssistant, unittest.TestCase):

    def test_auto_view(self):
        with traits_enaml.imports():
            from enaml.widgets.api import Window

        model = AllTypes()
        window = Window()
        window.insert_children(None, (auto_view(model=model),))
        with self.event_loop():
            window.show()
        self.check_component_counts(window)
        self.check_label_text(window)

    def test_auto_window(self):
        model = AllTypes()
        window = auto_window(model=model)
        with self.event_loop():
            window.show()
        self.check_component_counts(window)
        self.check_label_text(window)

    def check_component_counts(self, view):
        expected_counts = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 12]
        components = [
            'BoolEditor', 'ButtonEditor', 'DateEditor',
            'EnumEditor', 'FloatEditor', 'FloatRangeEditor',
            'IntEditor', 'IntRangeEditor', 'StrEditor',
            'TimeEditor', 'DefaultEditor', 'Label']
        for index, component in enumerate(components):
            items = self.find_all_enaml_widgets(view, component)
            self.assertEqual(len(items), expected_counts[index])

    def check_label_text(self, view):
        labels = [
            'Custom Bool Label:', 'Button Value', 'Date Value',
            'Enum Value', 'Float Value', 'Float Range Value',
            'Int Value', 'Int Range Value', 'Str Value',
            'Time Value', 'Traits Range Editor:', 'List Value']
        components = self.find_all_enaml_widgets(view, 'Label')
        self.assertEqual(len(components), 12)
        components_text = [component.text for component in components]
        self.assertItemsEqual(components_text, labels)



#----------------------------------------------------------------------------
#
#  Copyright (c) 2013-14, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#----------------------------------------------------------------------------
from enaml.core.dynamicscope import DynamicScope
from enaml.core.expression_engine import HandlerPair, ReadHandler
from enaml.core.funchelper import call_func
from enaml.core.operators import gen_tracer, op_notify, op_simple, op_update
from enaml.core.standard_handlers import HandlerMixin

from .traits_tracer import TraitsTracer


class TraitsTracedReadHandler(ReadHandler, HandlerMixin):
    """ An expression read handler which traces code execution.

    This handler is used in conjuction with the standard '<<' operator.

    """
    def __call__(self, owner, name):
        """ Evaluate and return the expression value.

        """
        func = self.func
        f_globals = func.func_globals
        f_builtins = f_globals['__builtins__']
        f_locals = self.get_locals(owner)
        tr = TraitsTracer(owner, name)
        scope = DynamicScope(owner, f_locals, f_globals, f_builtins, None, tr)
        return call_func(func, (tr,), {}, scope)


def trait_op_subscribe(code, scope_key, f_globals):
    """ The Traits Enaml operator function for the `<<` operator.

    This operator generates a tracer function with optimized local
    access and hooks it up to a TraitsTracedReadHandler. This
    operator does not support write semantics.

    Parameters
    ----------
    code : CodeType
        The code object created by the Enaml compiler.

    scope_key : object
        The block scope key created by the Enaml compiler.

    f_globals : dict
        The global scope for the for code execution.

    Returns
    -------
    result : HandlerPair
        A pair with the reader set to a TraitsTracedReadHandler.

    """
    func = gen_tracer(code, f_globals)
    reader = TraitsTracedReadHandler(func=func, scope_key=scope_key)
    return HandlerPair(reader=reader)


def trait_op_delegate(code, scope_key, f_globals):
    """ The Traits Enaml operator function for the `:=` operator.

    This operator combines the '<<' and the '>>' operators into a
    single operator. It supports both read and write semantics.

    Parameters
    ----------
    code : CodeType
        The code object created by the Enaml compiler.

    scope_key : object
        The block scope key created by the Enaml compiler.

    f_globals : dict
        The global scope for the for code execution.

    Returns
    -------
    result : HandlerPair
        A pair with the reader set to a TraitsTracedReadHandler and
        the writer set to a StandardInvertedWriteHandler.

    """
    p1 = trait_op_subscribe(code, scope_key, f_globals)
    p2 = op_update(code, scope_key, f_globals)
    return HandlerPair(reader=p1.reader, writer=p2.writer)


TRAIT_OPERATORS = {
    '=': op_simple,
    '::': op_notify,
    '>>': op_update,
    '<<': trait_op_subscribe,
    ':=': trait_op_delegate,
}



# Copyright (c) 2013 by Enthought Inc.
import StringIO
import sys
import unittest

import traits_enaml
from traits_enaml.testing.enaml_test_assistant import (
    EnamlTestAssistant, print_enaml_widget_tree)

ENAML_SOURCE = """\
from enaml.widgets.main_window import MainWindow
from traits_enaml.testing.tests.enaml_test_container import EnamlTestContainer

enamldef MainView(MainWindow):
    attr some_attr
    EnamlTestContainer:
        name = "test_container"
    EnamlTestContainer:
        name = "test_container 2"
    EnamlTestContainer:
        name = "test_container 3"
"""


class TestEnamlTestAssistant(unittest.TestCase):
    def setUp(self):
        self.test_assistant = EnamlTestAssistant()
        self.test_assistant.setUp()

    def tearDown(self):
        self.test_assistant.tearDown()

    def test_parse_and_create(self):
        # Check that parse_and_create is able to create enaml objects
        # which refer to other enaml files.
        assistant = self.test_assistant
        try:
            assistant.parse_and_create(ENAML_SOURCE)
        except ImportError:
            self.fail(msg="Failed to set up enaml class.")

    def test_parse_and_create_kwargs(self):
        # Test that parse_and_create correctly passes along kwargs.
        assistant = self.test_assistant
        attr_value = 123
        view, _ = assistant.parse_and_create(
            ENAML_SOURCE, some_attr=attr_value)

        with assistant.event_loop():
            main_view = assistant.find_enaml_widget(view, "MainView")
            self.assertEqual(attr_value, main_view.some_attr)

    def test_find_all_enaml_widgets(self):
        assistant = self.test_assistant

        with traits_enaml.imports():
            from traits_enaml.testing.tests.enaml_test_container import (
                EnamlTestContainer)

        view, _ = assistant.parse_and_create(ENAML_SOURCE)

        with assistant.event_loop():
            widgets = assistant.find_all_enaml_widgets(
                view, "EnamlTestContainer")

        self.assertEqual(len(widgets), 3)
        for widget in widgets:
            self.assertIsInstance(widget, EnamlTestContainer)


class TestEnamlTestHelperFunctions(EnamlTestAssistant, unittest.TestCase):

    def test_print_enaml_widget_tree(self):
        view, _ = self.parse_and_create(ENAML_SOURCE)
        stream = StringIO.StringIO()
        old_stdout = sys.stdout
        try:
            sys.stdout = stream
            print_enaml_widget_tree(view)
        finally:
            sys.stdout = old_stdout
            stream.close()
        lines = ''.join(stream.buflist).splitlines()
        # basic check we should have four items in the hierarchy.
        self.assertEqual(len(lines), 4)
